function [elval,elconv] = innerloop_EL(w)
%INNERLOOP_EL: calculates EL innerloop, w=f(theta)


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %these parameters could affect the calculation of the innerloop
    global el_max_innerloop_iterations el_printdiag 
    global el_max_step_len_dim el_tolerance_grad_direc

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    if isempty(el_printdiag) 
      el_printdiag = 0; %do not print information messages by default
    end

    if isempty(el_max_innerloop_iterations)
      el_max_innerloop_iterations = 100; %maximum number of iterations
           % IN the innerloop before it returns "Failed to converge". 
    end

    if ~isempty(el_max_step_len_dim )
      el_max_step_len_dim = 10^10; %maximum step length is 
         % el_max_step_len_dim*<number of momoments>
    end

    if isempty(el_tolerance_grad_direc)
      el_tolerance_grad_direc = .000000001; %stopping criterion value
                                            %if innerloop converges
    end

    el_min_step_mul = 0.01; %minimum multiplier of the 
    
    el_infinity = 10^100; %used in outerloop to determine
            %whether the numbers used in calculations are too big

    el_max_moment_val = 10^9; %if any number among moment values 
        %(i.e. in w) is bigger than this in absolute value, we report
        %divergence of the innerlooop. This is an important parameter that
        %depends on the problem. Making this too large may lead to
        %numerical instability of the innerloop optimization algorithm (the
        %'direc' matrix may be badly conditioned, which would lead to a bad
        %behavior of 'direc = direc\grad;' line below

    global el_prob el_lambda;

    if isbig(w,el_max_moment_val)
        elval=0;
        elconv=0;
        return
    end

    [n,cols] = size(w);  
    
    grad_tmp = zeros(n,cols);  

    lam = zeros(cols,1);
    testmax = .999999*n;
    iter = 1; 
    while iter <=el_max_innerloop_iterations;
        if el_printdiag ~= 0
            disp(['EL innerloop: iteration ' int2str(iter)]);
        end
        el_prob = 1./(1+w*lam);

        %The faster version:
        for j=1:cols
          grad_tmp(:,j) = el_prob.*w(:,j);
        end
        direc = grad_tmp'*grad_tmp;        
        grad = -sum(grad_tmp)';
    
        if isbig(direc,el_infinity)==0 && isbig(grad,el_infinity)==0
            if rcond(direc)<1e-15
              fprintf(1, 'EL: rcond = %g\n', rcond(direc));
            end
            direc = direc\grad;
            test = grad'*direc;  
        else
            test = testmax + 1; %convergence criteria too large
        end
        
        direc_len = sqrt(sum(direc.^2));
        if direc_len > el_max_step_len_dim*cols 
            direc = direc/direc_len*el_max_step_len_dim*cols;
        end

        if test > testmax
            if el_printdiag ~= 0
                disp('EL innerloop: convergence criteria too large');
            end
            elval = 0;
            elconv = 0;            
            return
        end
    
        if test <= el_tolerance_grad_direc;
            elval = - 2*sum(log(el_prob));
            el_lambda = lam;
            elconv = 1;
            return
        end

        lik = sum(log(el_prob));
        step = 1;        
        improved = 0;

        while step >= el_min_step_mul && improved == 0;
            lamtry = lam - step*direc;
            el_prob = 1.0 + (w*lamtry);        
            if any(el_prob <= 0) == 0 && sum(-log(el_prob)) < lik;
                %checked positivity and the criterion got smaller
                improved = 1;
                lam = lamtry;
            else
                step = step/2;
            end           
        end;
        if improved==0
            if el_printdiag ~= 0
                disp('EL innerloop: stepsize failed to find increase');
            end
            elval = 0;
            elconv = 0;   
            return
        end        
        iter = iter + 1;
    end

    if el_printdiag ~= 0
        disp('EL innerloop: Maximum iterations reached');
    end

    elval = 0;
    elconv = 0;
end
